#include "Matrix.h"

using namespace std;

Matrix::Matrix(int nRows, int nCols) throw(invalid_argument) : nRows(nRows), nCols(nCols)
  {
    if (nRows <= 0 || nCols <= 0)
        throw invalid_argument("Matrix dimensions must be greater than 0");

    allocMatrix();

    for (int i = 0;  i < nRows;  ++i)
      {
        for (int j = 0;  j < nCols;  ++j)
            p[i][j] = 0.0;
      }
  }

Matrix::Matrix(const Matrix& m) : nRows(m.nRows), nCols(m.nCols)
  {
    allocMatrix();
    for (int i = 0; i <nRows; ++i)
      {
        for (int j = 0;  j <nCols;  ++j)
            p[i][j] = m.p[i][j];
      }
  }

Matrix::~Matrix()
  {
    for (int i = 0;  i < nRows; i++)
        delete[] p[i];

    delete[] p;
    p = 0;
  }

void Matrix::allocMatrix()
  {
    p = new double*[nRows];
    for (int i = 0;  i < nRows; ++i)
        p[i] = new double[nCols];
  }

Matrix& Matrix::operator=(const Matrix& m)
  {
    if (this == &m)
        return *this;  // avoid self-assignment
    else
      {
        if (nRows != m.nRows || nCols != m.nCols)
          {
            this->~Matrix();
            nRows = m.nRows;
            nCols = m.nCols;
            allocMatrix();
          }

        for (int i = 0;  i < nRows;  ++i)
          {
            for (int j = 0;  j < nCols;  ++j)
                p[i][j] = m.p[i][j];
          }
      }

    return *this;
  }

Matrix operator+(const Matrix& m1, const Matrix& m2) throw(invalid_argument)
  {
    if ((m1.nRows != m2.nRows) || (m1.nCols != m2.nCols))
        throw invalid_argument("matrices not compatible for addition");

    Matrix result(m1.nRows, m1.nCols);

    for (int i = 0;  i < m1.nRows;  ++i)
      {
        for (int j = 0;  j < m1.nCols;  ++j)
            result.p[i][j] = m1.p[i][j] + m2.p[i][j];
      }

    return result;
  }

Matrix operator-(const Matrix& m1, const Matrix& m2) throw(invalid_argument)
  {
    if ((m1.nRows != m2.nRows) || (m1.nCols != m2.nCols))
        throw invalid_argument("matrices not compatible for addition");

    Matrix result(m1.nRows, m1.nCols);

    for (int i = 0;  i < m1.nRows;  ++i)
      {
        for (int j = 0;  j < m1.nCols;  ++j)
            result.p[i][j] = m1.p[i][j] - m2.p[i][j];
      }

    return result;
  }

Matrix operator*(const Matrix& m1, const Matrix& m2) throw(invalid_argument)
  {
    if (!Matrix::checkArgs(m1, m2))
        throw invalid_argument("matrices not compatible for multiplication");

    Matrix result(m1.nRows, m2.nCols);

    for (int i = 0;  i < result.nRows;  ++i)
      {
        for (int j = 0;  j < result.nCols;  ++j)
          {
            double sum = 0.0;

            for (int k = 0;  k < m1.nCols;  ++k)
                sum += m1.p[i][k]*m2.p[k][j];

            result.p[i][j] = sum;
          }
      }

    return result;
  }

Matrix operator*(double x, const Matrix& m)
  {
    Matrix result(m.nRows, m.nCols);

    for (int i = 0;  i < m.nRows;  ++i)
      {
        for (int j = 0;  j < m.nCols;  ++j)
            result.p[i][j] = x*m.p[i][j];
      }

    return result;
  }

Matrix operator*(const Matrix& m, double x)
  {
    return x*m;
  }

ostream &operator<<(ostream &out, const Matrix &m)
  {
    for (int i = 0;  i < m.nRows;  ++i)
      {
        for (int j = 0;  j < m.nCols;  ++j)
            out << m.p[i][j] << "  ";
        out << endl;
      }

    return out;
  }

bool Matrix::checkArgs(Matrix m1, Matrix m2)
  {
    // check that matrices are compatible for multiplication
    return m1.nCols == m2.nRows;
  }
