#if !defined(STOP_WATCH_H)
#define STOP_WATCH_H

#include <ctime>

using namespace std;


/**
 * A simple "stopwatch" timer class with millisecond accuracy.<br>
 * Usage Notes:<br>
 * 1.  A stopwatch is started automatically when constructed.<br>
 * 2.  Method start() restarts the stopwatch.<br>
 * 3.  Method stop() can only be called if the stopwatch is currently running
 *     (i.e., if the stopwatch is not already stopped).<br>
 * 4.  A single stopwatch can be restarted/stopped multiple times.<br>
 * 5.  Methods that return the elapsed time can only be called if the
 *     stopwatch is stopped.<br>
 */
class StopWatch
  {
    public:

        /**
         * Creates and starts the StopWatch.  Subsequent calls
         * to start simply reset the start time.
         */
        StopWatch();

        /**
         * Sets the start time for the StopWatch to the current time.
         */
        void start();

        /**
         * Sets the stop time for the StopWatch to the current time.
         */
        void stop();

        /**
         * Returns the elapsed time in milliseconds
         */
        long getElapsedTimeInMilliseconds();

        /**
         * Returns the elapsed time in seconds
         */
        double getElapsedTimeInSeconds();

        /**
         * Returns the elapsed time in minutes
         */
        double getElapsedTimeInMinutes();

        /**
         * Returns the elapsed time in hours
         */
        double getElapsedTimeInHours();

        /**
         * Returns the start time in milliseconds
         */
        long getStartTime();

        /**
         * Returns the stop time in milliseconds
         */
        long getStopTime();

    private:

        clock_t startTime;
        clock_t stopTime;

        bool isRunning;
  };

#endif
